@@ -9,7 +9,7 @@ import (
 	"github.com/sourcegraph/jsonrpc2"
 )
 
-func (h *langHandler) handleInitialize(ctx context.Context, conn *jsonrpc2.Conn, req *jsonrpc2.Request) (result interface{}, err error) {
+func (h *langHandler) handleInitialize(_ context.Context, conn *jsonrpc2.Conn, req *jsonrpc2.Request) (result any, err error) {
 	if req.Params == nil {
 		return nil, &jsonrpc2.Error{Code: jsonrpc2.CodeInvalidParams}
 	}
@@ -33,11 +33,12 @@ func (h *langHandler) handleInitialize(ctx context.Context, conn *jsonrpc2.Conn,
 	}
 
 	var completion *CompletionProvider
-	var hasCompletionCommand bool = params.InitializationOptions.Completion
-	var hasHoverCommand bool = params.InitializationOptions.Hover
-	var hasCodeActionCommand bool = params.InitializationOptions.CodeAction
-	var hasSymbolCommand bool = params.InitializationOptions.DocumentSymbol
-	var hasFormatCommand bool = params.InitializationOptions.DocumentFormatting
+	var hasCompletionCommand bool
+	var hasHoverCommand bool
+	var hasCodeActionCommand bool
+	var hasSymbolCommand bool
+	var hasFormatCommand bool
+	var hasRangeFormatCommand bool
 	var hasDefinitionCommand bool
 
 	if len(h.commands) > 0 {
@@ -61,10 +62,22 @@ func (h *langHandler) handleInitialize(ctx context.Context, conn *jsonrpc2.Conn,
 			}
 			if v.FormatCommand != "" {
 				hasFormatCommand = true
+				if v.FormatCanRange {
+					hasRangeFormatCommand = true
+				}
 			}
 		}
 	}
 
+	if params.InitializationOptions != nil {
+		hasCompletionCommand = params.InitializationOptions.Completion
+		hasHoverCommand = params.InitializationOptions.Hover
+		hasCodeActionCommand = params.InitializationOptions.CodeAction
+		hasSymbolCommand = params.InitializationOptions.DocumentSymbol
+		hasFormatCommand = params.InitializationOptions.DocumentFormatting
+		hasRangeFormatCommand = params.InitializationOptions.RangeFormatting
+	}
+
 	if hasCompletionCommand {
 		chars := []string{"."}
 		if len(h.triggerChars) > 0 {
@@ -79,6 +92,7 @@ func (h *langHandler) handleInitialize(ctx context.Context, conn *jsonrpc2.Conn,
 		Capabilities: ServerCapabilities{
 			TextDocumentSync:           TDSKFull,
 			DocumentFormattingProvider: hasFormatCommand,
+			RangeFormattingProvider:    hasRangeFormatCommand,
 			DocumentSymbolProvider:     hasSymbolCommand,
 			DefinitionProvider:         hasDefinitionCommand,
 			CompletionProvider:         completion,
