@@ -4,7 +4,6 @@ import (
 	"bytes"
 	"context"
 	"fmt"
-	"io/ioutil"
 	"log"
 	"net/url"
 	"os"
@@ -13,15 +12,30 @@ import (
 	"runtime"
 	"strconv"
 	"strings"
+	"sync"
 	"time"
 	"unicode"
 	"unicode/utf16"
 
-	"github.com/mattn/go-unicodeclass"
 	"github.com/reviewdog/errorformat"
 	"github.com/sourcegraph/jsonrpc2"
+
+	"github.com/mattn/go-unicodeclass"
 )
 
+type eventType int
+
+const (
+	eventTypeChange eventType = iota
+	eventTypeSave
+	eventTypeOpen
+)
+
+type lintRequest struct {
+	URI       DocumentURI
+	EventType eventType
+}
+
 // Config is
 type Config struct {
 	Version        int                    `yaml:"version"`
@@ -30,7 +44,7 @@ type Config struct {
 	Commands       *[]Command             `yaml:"commands"        json:"commands"`
 	Languages      *map[string][]Language `yaml:"languages"       json:"languages"`
 	RootMarkers    *[]string              `yaml:"root-markers"    json:"rootMarkers"`
-	TriggerChars   []string              `yaml:"trigger-chars"   json:"triggerChars"`
+	TriggerChars   []string               `yaml:"trigger-chars"   json:"triggerChars"`
 	LintDebounce   Duration               `yaml:"lint-debounce"   json:"lintDebounce"`
 	FormatDebounce Duration               `yaml:"format-debounce" json:"formatDebounce"`
 
@@ -62,7 +76,9 @@ type Language struct {
 	LintSource         string            `yaml:"lint-source" json:"lintSource"`
 	LintSeverity       int               `yaml:"lint-severity" json:"lintSeverity"`
 	LintWorkspace      bool              `yaml:"lint-workspace" json:"lintWorkspace"`
+	LintOnSave         bool              `yaml:"lint-on-save" json:"lintOnSave"`
 	FormatCommand      string            `yaml:"format-command" json:"formatCommand"`
+	FormatCanRange     bool              `yaml:"format-can-range" json:"formatCanRange"`
 	FormatStdin        bool              `yaml:"format-stdin" json:"formatStdin"`
 	SymbolCommand      string            `yaml:"symbol-command" json:"symbolCommand"`
 	SymbolStdin        bool              `yaml:"symbol-stdin" json:"symbolStdin"`
@@ -92,7 +108,7 @@ func NewHandler(config *Config) jsonrpc2.Handler {
 		configs:           *config.Languages,
 		provideDefinition: config.ProvideDefinition,
 		files:             make(map[DocumentURI]*File),
-		request:           make(chan DocumentURI),
+		request:           make(chan lintRequest),
 		lintDebounce:      time.Duration(config.LintDebounce),
 		lintTimer:         nil,
 
@@ -110,13 +126,14 @@ func NewHandler(config *Config) jsonrpc2.Handler {
 }
 
 type langHandler struct {
+	mu                sync.Mutex
 	loglevel          int
 	logger            *log.Logger
 	commands          []Command
 	configs           map[string][]Language
 	provideDefinition bool
 	files             map[DocumentURI]*File
-	request           chan DocumentURI
+	request           chan lintRequest
 	lintDebounce      time.Duration
 	lintTimer         *time.Timer
 	formatDebounce    time.Duration
@@ -212,14 +229,14 @@ func toURI(path string) DocumentURI {
 	}).String())
 }
 
-func (h *langHandler) lintRequest(uri DocumentURI) {
+func (h *langHandler) lintRequest(uri DocumentURI, eventType eventType) {
 	if h.lintTimer != nil {
 		h.lintTimer.Reset(h.lintDebounce)
 		return
 	}
 	h.lintTimer = time.AfterFunc(h.lintDebounce, func() {
 		h.lintTimer = nil
-		h.request <- uri
+		h.request <- lintRequest{URI: uri, EventType: eventType}
 	})
 }
 
@@ -237,21 +254,21 @@ func (h *langHandler) linter() {
 	running := make(map[DocumentURI]context.CancelFunc)
 
 	for {
-		uri, ok := <-h.request
+		lintReq, ok := <-h.request
 		if !ok {
 			break
 		}
 
-		cancel, ok := running[uri]
+		cancel, ok := running[lintReq.URI]
 		if ok {
 			cancel()
 		}
 
 		ctx, cancel := context.WithCancel(context.Background())
-		running[uri] = cancel
+		running[lintReq.URI] = cancel
 
 		go func() {
-			uriToDiagnostics, err := h.lint(ctx, uri)
+			uriToDiagnostics, err := h.lint(ctx, lintReq.URI, lintReq.EventType)
 			if err != nil {
 				h.logger.Println(err)
 				return
@@ -259,11 +276,11 @@ func (h *langHandler) linter() {
 
 			for diagURI, diagnostics := range uriToDiagnostics {
 				if diagURI == "file:" {
-					diagURI = uri
+					diagURI = lintReq.URI
 				}
 				version := 0
-				if _, ok := h.files[uri]; ok {
-					version = h.files[uri].Version
+				if _, ok := h.files[lintReq.URI]; ok {
+					version = h.files[lintReq.URI].Version
 				}
 				h.conn.Notify(
 					ctx,
@@ -282,7 +299,7 @@ func matchRootPath(fname string, markers []string) string {
 	dir := filepath.Dir(filepath.Clean(fname))
 	var prev string
 	for dir != prev {
-		files, _ := ioutil.ReadDir(dir)
+		files, _ := os.ReadDir(dir)
 		for _, file := range files {
 			name := file.Name()
 			isDir := file.IsDir()
@@ -338,7 +355,7 @@ func isFilename(s string) bool {
 	}
 }
 
-func (h *langHandler) lint(ctx context.Context, uri DocumentURI) (map[DocumentURI][]Diagnostic, error) {
+func (h *langHandler) lint(ctx context.Context, uri DocumentURI, eventType eventType) (map[DocumentURI][]Diagnostic, error) {
 	f, ok := h.files[uri]
 	if !ok {
 		return nil, fmt.Errorf("document not found: %v", uri)
@@ -357,6 +374,10 @@ func (h *langHandler) lint(ctx context.Context, uri DocumentURI) (map[DocumentUR
 			if dir := matchRootPath(fname, cfg.RootMarkers); dir == "" && cfg.RequireMarker == true {
 				continue
 			}
+			// if LintOnSave is true, only lint when running on didSave
+			if cfg.LintOnSave && eventType != eventTypeSave {
+				continue
+			}
 			if cfg.LintCommand != "" {
 				configs = append(configs, cfg)
 			}
@@ -570,7 +591,7 @@ func (h *langHandler) closeFile(uri DocumentURI) error {
 }
 
 func (h *langHandler) saveFile(uri DocumentURI) error {
-	h.lintRequest(uri)
+	h.lintRequest(uri, eventTypeSave)
 	return nil
 }
 
@@ -584,7 +605,7 @@ func (h *langHandler) openFile(uri DocumentURI, languageID string, version int)
 	return nil
 }
 
-func (h *langHandler) updateFile(uri DocumentURI, text string, version *int) error {
+func (h *langHandler) updateFile(uri DocumentURI, text string, version *int, eventType eventType) error {
 	f, ok := h.files[uri]
 	if !ok {
 		return fmt.Errorf("document not found: %v", uri)
@@ -594,7 +615,7 @@ func (h *langHandler) updateFile(uri DocumentURI, text string, version *int) err
 		f.Version = *version
 	}
 
-	h.lintRequest(uri)
+	h.lintRequest(uri, eventType)
 	return nil
 }
 
@@ -624,7 +645,7 @@ func (h *langHandler) addFolder(folder string) {
 	}
 }
 
-func (h *langHandler) handle(ctx context.Context, conn *jsonrpc2.Conn, req *jsonrpc2.Request) (result interface{}, err error) {
+func (h *langHandler) handle(ctx context.Context, conn *jsonrpc2.Conn, req *jsonrpc2.Request) (result any, err error) {
 	switch req.Method {
 	case "initialize":
 		return h.handleInitialize(ctx, conn, req)
@@ -642,6 +663,8 @@ func (h *langHandler) handle(ctx context.Context, conn *jsonrpc2.Conn, req *json
 		return h.handleTextDocumentDidClose(ctx, conn, req)
 	case "textDocument/formatting":
 		return h.handleTextDocumentFormatting(ctx, conn, req)
+	case "textDocument/rangeFormatting":
+		return h.handleTextDocumentRangeFormatting(ctx, conn, req)
 	case "textDocument/documentSymbol":
 		return h.handleTextDocumentSymbol(ctx, conn, req)
 	case "textDocument/completion":
@@ -656,6 +679,8 @@ func (h *langHandler) handle(ctx context.Context, conn *jsonrpc2.Conn, req *json
 		return h.handleWorkspaceExecuteCommand(ctx, conn, req)
 	case "workspace/didChangeConfiguration":
 		return h.handleWorkspaceDidChangeConfiguration(ctx, conn, req)
+	case "workspace/didChangeWorkspaceFolders":
+		return h.handleDidChangeWorkspaceWorkspaceFolders(ctx, conn, req)
 	case "workspace/workspaceFolders":
 		return h.handleWorkspaceWorkspaceFolders(ctx, conn, req)
 	}
@@ -674,3 +699,10 @@ func replaceCommandInputFilename(command, fname, rootPath string) string {
 
 	return command
 }
+
+func succeeded(err error) bool {
+	exitErr, ok := err.(*exec.ExitError)
+	// When the context is canceled, the process is killed,
+	// and the exit code is -1
+	return ok && exitErr.ExitCode() < 0
+}
