@@ -4,21 +4,21 @@ import (
 	"context"
 	"flag"
 	"fmt"
-	"io/ioutil"
+	"io"
 	"log"
 	"os"
 	"path/filepath"
 	"runtime"
 
+	"github.com/sourcegraph/jsonrpc2"
 	"gopkg.in/yaml.v3"
 
 	"github.com/mattn/efm-langserver/langserver"
-	"github.com/sourcegraph/jsonrpc2"
 )
 
 const (
 	name    = "efm-langserver"
-	version = "0.0.44"
+	version = "0.0.48"
 )
 
 var revision = "HEAD"
@@ -45,15 +45,21 @@ func main() {
 	}
 
 	if yamlfile == "" {
-		dir := os.Getenv("HOME")
-		if dir == "" && runtime.GOOS == "windows" {
-			dir = filepath.Join(os.Getenv("APPDATA"), "efm-langserver")
+		var configHome string
+		if runtime.GOOS == "windows" {
+			configHome = os.Getenv("APPDATA")
 		} else {
-			dir = filepath.Join(dir, ".config", "efm-langserver")
+			configHome = os.Getenv("XDG_CONFIG_HOME")
+			if configHome == "" {
+				configHome = filepath.Join(os.Getenv("HOME"), ".config")
+			}
 		}
-		if err := os.MkdirAll(dir, 0700); err != nil {
+
+		dir := filepath.Join(configHome, "efm-langserver")
+		if err := os.MkdirAll(dir, 0o700); err != nil {
 			log.Fatal(err)
 		}
+
 		yamlfile = filepath.Join(dir, "config.yaml")
 	} else {
 		_, err := os.Stat(yamlfile)
@@ -81,7 +87,7 @@ func main() {
 	}
 
 	if quiet {
-		log.SetOutput(ioutil.Discard)
+		log.SetOutput(io.Discard)
 	}
 
 	log.Println("efm-langserver: reading on stdin, writing on stdout")
@@ -96,7 +102,7 @@ func main() {
 	var connOpt []jsonrpc2.ConnOpt
 
 	if logfile != "" {
-		f, err := os.OpenFile(logfile, os.O_CREATE|os.O_RDWR|os.O_APPEND, 0660)
+		f, err := os.OpenFile(logfile, os.O_CREATE|os.O_RDWR|os.O_APPEND, 0o660)
 		if err != nil {
 			log.Fatal(err)
 		}
@@ -108,7 +114,7 @@ func main() {
 	}
 
 	if quiet && (logfile == "" || loglevel < 5) {
-		connOpt = append(connOpt, jsonrpc2.LogMessages(log.New(ioutil.Discard, "", 0)))
+		connOpt = append(connOpt, jsonrpc2.LogMessages(log.New(io.Discard, "", 0)))
 	}
 
 	handler := langserver.NewHandler(config)
